@@ -1,28 +1,28 @@
-use proc_macro2::{Delimiter, LineColumn, Spacing, Span, TokenStream, TokenTree};
-use quote::quote;
-use std::collections::BTreeSet;
+use proc_macro2::{Delimiter, Ident, LineColumn, Spacing, Span, TokenStream, TokenTree};
+use quote::quote_spanned;
+use std::collections::BTreeMap;
 use std::fmt::Write;
 
 pub struct EmbedPython {
 	pub python: String,
-	pub variables: TokenStream,
-	pub variable_names: BTreeSet<String>,
+	pub variables: BTreeMap<String, Ident>,
 	pub first_indent: Option<usize>,
 	pub loc: LineColumn,
+	pub compile_time: bool,
 }
 
 impl EmbedPython {
 	pub fn new() -> Self {
 		Self {
 			python: String::new(),
-			variables: TokenStream::new(),
-			variable_names: BTreeSet::new(),
+			variables: BTreeMap::new(),
 			loc: LineColumn { line: 1, column: 0 },
 			first_indent: None,
+			compile_time: false,
 		}
 	}
 
-	fn add_whitespace(&mut self, span: Span, loc: LineColumn) -> syn::Result<()> {
+	fn add_whitespace(&mut self, span: Span, loc: LineColumn) -> Result<(), TokenStream> {
 		if loc.line > self.loc.line {
 			while loc.line > self.loc.line {
 				self.python.push('\n');
@@ -30,7 +30,7 @@ impl EmbedPython {
 			}
 			let first_indent = *self.first_indent.get_or_insert(loc.column);
 			let indent = loc.column.checked_sub(first_indent);
-			let indent = indent.ok_or_else(|| error!(span, "Invalid indentation on line {}", loc.line))?;
+			let indent = indent.ok_or_else(|| quote_spanned!(span => compile_error!{"Invalid indentation"}))?;
 			for _ in 0..indent {
 				self.python.push(' ');
 			}
@@ -45,7 +45,7 @@ impl EmbedPython {
 		Ok(())
 	}
 
-	pub fn add(&mut self, input: TokenStream) -> syn::Result<()> {
+	pub fn add(&mut self, input: TokenStream) -> Result<(), TokenStream> {
 		let mut tokens = input.into_iter();
 
 		while let Some(token) = tokens.next() {
@@ -70,7 +70,7 @@ impl EmbedPython {
 					self.loc.column += end.len();
 				}
 				TokenTree::Punct(x) => {
-					if x.as_char() == '\'' && x.spacing() == Spacing::Joint {
+					if !self.compile_time && x.as_char() == '\'' && x.spacing() == Spacing::Joint {
 						let name = if let Some(TokenTree::Ident(name)) = tokens.next() {
 							name
 						} else {
@@ -79,12 +79,7 @@ impl EmbedPython {
 						let name_str = format!("_RUST_{}", name);
 						self.python.push_str(&name_str);
 						self.loc.column += name_str.chars().count() - 6 + 1;
-						if self.variable_names.insert(name_str.clone()) {
-							self.variables.extend(quote! {
-								globals.set_item(#name_str, #name)
-									.expect("Unable to convert variable to Python");
-							});
-						}
+						self.variables.entry(name_str).or_insert(name);
 					} else if x.as_char() == '#' && x.spacing() == Spacing::Joint {
 						// Convert '##' to '//', because otherwise it's
 						// impossible to use the Python operators '//' and '//='.
